library proc_extend;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Extend Process Module for Lightning.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

{$IMAGEBASE $413A0000}

uses
  SysUtils,
  Windows,
  Classes,
  StdCtrls,
  Spin,
  Graphics,
  common in '..\common.pas';

{$R *.RES}

type
  TDetails = class(TObject)
    gbDetails:    TGroupBox;
    lExtendTime:  TLabel;
    seExtendTime: TSpinEdit;
    procedure seExtendTimeChange(Sender: TObject);
  public
    pulsed:   Boolean;
    lastTime: Int64;
    extend:   Cardinal;
  end;

procedure TDetails.seExtendTimeChange(Sender: TObject);
begin
  extend := seExtendTime.Value;
end;

function init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  Result := return('PExtend|Pulse Control', name, nameLength);
  tickInit();
end;

procedure finish(); cdecl;
begin
  //
end;

function create(controlHandle: THandle): PLightningModule; cdecl;
var
  myDetails: TDetails;
begin
  New(Result);
  with Result^ do
  begin
    handle        := controlHandle;
    title         := 'Extend';
    settings      := True;
    width         := -1;
    height        := -1;
    inputTop      := -1;
    infoTop       := -1;
    outputTop     := -1;
    inputs        := 1;
    outputs       := 1;
    inputTooltip  := 'Status';
    outputTooltip := 'Extended status';
    myDetails     := TDetails.Create;
    reserved      := nil;
    details       := myDetails;
  end;
  with myDetails do
  begin
    lastTime := 0;
    pulsed   := False;
    extend   := 50;
  end;
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
begin
  TDetails(lightningModule^.details).Free;
  Dispose(lightningModule);
end;

function action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
var
  details: TDetails;
  outcome: Boolean;
begin
  details := TDetails(lightningModule.details);
  if spectrumAverage(bridge, fftMin, fftMax) > enabledBoundary then
  begin
    details.pulsed := True;
    outcome := True;
  end
  else
  begin
    if details.pulsed then
    begin
      details.pulsed   := False;
      details.lastTime := tickCount;
    end;
    outcome := tickCount - details.lastTime <= details.extend;
  end;
  if outcome then
    createSpectrum(bridge, levelMax)
  else
    createSpectrum(bridge, levelMin);
  Result := True;
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
begin
  with TDetails(lightningModule.details) do
  begin
    gbDetails              := TGroupBox.Create(nil);
    gbDetails.Top          := detailsTop;
    gbDetails.Left         := detailsLeft;
    gbDetails.Width        := detailsWidth;
    gbDetails.Height       := detailsHeight;
    gbDetails.Ctl3D        := True;
    gbDetails.Color        := clBtnFace;
    gbDetails.Caption      := 'Extend Settings';
    lExtendTime            := TLabel.Create(gbDetails);
    lExtendTime.AutoSize   := True;
    lExtendTime.Top        := 20;
    lExtendTime.Left       := 8;
    lExtendTime.Caption    := 'Extend time (ms):';
    lExtendTime.Parent     := gbDetails;
    seExtendTime           := TSpinEdit.Create(gbDetails);
    seExtendTime.Top       := 40;
    seExtendTime.Left      := 8;
    seExtendTime.MinValue  := 0;
    seExtendTime.MaxValue  := 10000;
    seExtendTime.Increment := 10;
    seExtendTime.Value     := extend;
    seExtendTime.OnChange  := seExtendTimeChange;
    seExtendTime.Parent    := gbDetails;
    gbDetails.ParentWindow := handle;
  end;
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  with TDetails(lightningModule.details) do
  begin
    seExtendTime.Free;
    lExtendTime.Free;
    gbDetails.Free;
  end;
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
var
  bString: string;
begin
  bString := settings;
  bString := parse(bString, ',');
  if bString <> '' then
    TDetails(lightningModule.details).extend := StrToInt(bString);
end;

function save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
begin
  Result := return(IntToStr(TDetails(lightningModule.details).extend), settings, settingsLength);
end;

exports
  init, finish, create, destroy, action, show, hide, open, save;

end.
